@@ -17,6 +17,7 @@ from .base import get_is_adversarial
 from .base import raise_if_kwargs
 
 from ..external.clipping_aware_rescaling import l2_clipping_aware_rescaling
+from .base import verify_input_bounds
 
 
 class BaseAdditiveNoiseAttack(FixedEpsilonAttack, ABC):
@@ -33,6 +34,8 @@ class BaseAdditiveNoiseAttack(FixedEpsilonAttack, ABC):
         x, restore_type = ep.astensor_(inputs)
         del inputs, criterion, kwargs
 
+        verify_input_bounds(x, model)
+
         min_, max_ = model.bounds
         p = self.sample_noise(x)
         epsilons = self.get_epsilons(x, p, epsilon, min_=min_, max_=max_)
@@ -164,6 +167,8 @@ class BaseRepeatedAdditiveNoiseAttack(FixedEpsilonAttack, ABC):
         criterion_ = get_criterion(criterion)
         del inputs, criterion, kwargs
 
+        verify_input_bounds(x0, model)
+
         is_adversarial = get_is_adversarial(criterion_, model)
 
         min_, max_ = model.bounds
