@@ -130,6 +130,8 @@ class Repeated(AttackWithDistance):
         x, restore_type = ep.astensor_(inputs)
         del inputs
 
+        verify_input_bounds(x, model)
+
         criterion = get_criterion(criterion)
 
         was_iterable = True
@@ -252,6 +254,8 @@ class FixedEpsilonAttack(AttackWithDistance):
         x, restore_type = ep.astensor_(inputs)
         del inputs
 
+        verify_input_bounds(x, model)
+
         criterion = get_criterion(criterion)
         is_adversarial = get_is_adversarial(criterion, model)
 
@@ -389,6 +393,8 @@ class MinimizationAttack(AttackWithDistance):
         x, restore_type = ep.astensor_(inputs)
         del inputs
 
+        verify_input_bounds(x, model)
+
         criterion = get_criterion(criterion)
         is_adversarial = get_is_adversarial(criterion, model)
 
@@ -485,3 +491,9 @@ def raise_if_kwargs(kwargs: Dict[str, Any]) -> None:
         raise TypeError(
             f"attack got an unexpected keyword argument '{next(iter(kwargs.keys()))}'"
         )
+
+
+def verify_input_bounds(input: ep.Tensor, model: Model) -> None:
+    # verify that input to the attack lies within model's input bounds
+    assert input.min().item() >= model.bounds.lower
+    assert input.max().item() <= model.bounds.upper
