@@ -19,6 +19,7 @@ from .base import MinimizationAttack
 from .base import T
 from .base import get_criterion
 from .base import raise_if_kwargs
+from .base import verify_input_bounds
 
 from .gradient_descent_base import AdamOptimizer
 
@@ -75,6 +76,8 @@ class L2CarliniWagnerAttack(MinimizationAttack):
         criterion_ = get_criterion(criterion)
         del inputs, criterion, kwargs
 
+        verify_input_bounds(x, model)
+
         N = len(x)
 
         if isinstance(criterion_, Misclassification):
