@@ -21,10 +21,11 @@ from .base import MinimizationAttack, get_is_adversarial
 from .base import get_criterion
 from .base import T
 from .base import raise_if_kwargs
+from .base import verify_input_bounds
 from ..distances import l2, linf
 
 
-class HopSkipJump(MinimizationAttack):
+class HopSkipJumpAttack(MinimizationAttack):
     """A powerful adversarial attack that requires neither gradients
     nor probabilities [#Chen19].
 
@@ -101,6 +102,8 @@ class HopSkipJump(MinimizationAttack):
         originals, restore_type = ep.astensor_(inputs)
         del inputs, kwargs
 
+        verify_input_bounds(originals, model)
+
         criterion = get_criterion(criterion)
         is_adversarial = get_is_adversarial(criterion, model)
 
@@ -251,7 +254,7 @@ class HopSkipJump(MinimizationAttack):
         perturbed = ep.expand_dims(x_advs, 0) + scaled_rv
         perturbed = ep.clip(perturbed, 0, 1)
 
-        rv = (perturbed - x_advs) / atleast_kd(ep.expand_dims(delta, 0), rv.ndim)
+        rv = (perturbed - x_advs) / atleast_kd(ep.expand_dims(delta + 1e-8, 0), rv.ndim)
 
         multipliers_list: List[ep.Tensor] = []
         for step in range(steps):
@@ -259,8 +262,22 @@ class HopSkipJump(MinimizationAttack):
             multipliers_list.append(
                 ep.where(
                     decision,
-                    ep.ones(x_advs, (len(x_advs,))),
-                    -ep.ones(x_advs, (len(decision,))),
+                    ep.ones(
+                        x_advs,
+                        (
+                            len(
+                                x_advs,
+                            )
+                        ),
+                    ),
+                    -ep.ones(
+                        x_advs,
+                        (
+                            len(
+                                decision,
+                            )
+                        ),
+                    ),
                 )
             )
         # (steps, bs, ...)
