@@ -22,7 +22,9 @@ def rotate_and_shift(
 
 
 def transform_pt(
-    x_e: Tensor, translation: Tuple[float, float] = (0.0, 0.0), rotation: float = 0.0,
+    x_e: Tensor,
+    translation: Tuple[float, float] = (0.0, 0.0),
+    rotation: float = 0.0,
 ) -> Any:
     import torch
 
@@ -45,7 +47,12 @@ def transform_pt(
     assert len(x.shape) == 4
     assert theta.shape[1:] == (2, 3)
 
-    bs, _, n_x, n_y, = x.shape
+    (
+        bs,
+        _,
+        n_x,
+        n_y,
+    ) = x.shape
 
     def create_meshgrid(x: torch.Tensor) -> torch.Tensor:
         space_x = torch.linspace(-1, 1, n_x, device=x.device)
@@ -72,7 +79,9 @@ def transform_pt(
 # https://github.com/kevinzakka/spatial-transformer-network/blob/master/stn/transformer.py
 # state @375f990 on 3 Jun 2018
 def transform_tf(
-    x_e: Tensor, translation: Tuple[float, float] = (0.0, 0.0), rotation: float = 0.0,
+    x_e: Tensor,
+    translation: Tuple[float, float] = (0.0, 0.0),
+    rotation: float = 0.0,
 ) -> Any:
     """
     Input
